/*
 * Name:    vra_relation_update_Oracle.sql
 *
 * Purpose: Corrects the Inscribe source write back issue for data already
 *          cataloged.
 *          Uses IREntityChanges and DTValueToEntityMap to determine the
 *          missing mapping information in the source data, then inserts the
 *          missing relationships.
 *
 * Date:    2004/02/27
 *
 * Copyright (c) 2004, Luna Imaging, Inc.  All Rights Reserved
 *
*/

set echo off
set termout off
set trimspool on
set feedback off
set linesize 500
set wrap off

set serveroutput on size 1000000;

spool vra_relation_update_cmd.sql

Declare
cursor missingMapping is
  /*
   * Determine the list of EntityType/Entities matching the following criteria:
   * 1) Dependent EntityType Entities
   * 2) Have been committed for the source data
   * 3) Dependent EntityTypes which only contain a single mapping field
   *    (i.e. Notes)
  */
  Select EntityTypeID, EntityID
  from IREntityChanges EC1
  where changecategory = 2
  AND committedtosourcedata = 1
  AND changetype = 1
  AND entityid not in (select distinct entityid 
                       from IREntityChanges EC2
                       where (changecategory, changetype) IN ((1,1),(4,2))
                       and EC2.entitytypeid = EC1.entitytypeid)
  AND 1 = (select count(*) from ISEntityFields, ISEntityTypes 
           where ISEntityFields.EntityTypeID = ISEntityTypes.EntityTypeID 
           and ISEntityTypes.IsDependent = 1 
           and ISEntityFields.EntityTypeID = EC1.EntityTypeID)
  Group by entitytypeid, entityid
  ORDER BY entitytypeid, entityid; 
mapping missingMapping%ROWTYPE;

vFieldID           number;

vSourceTable       varchar2(255);
vSourceColumnID    varchar2(255);
vSourceColumnField varchar2(255);

Begin

  dbms_output.put_line(chr(10) || chr(10) || chr(10));
  dbms_output.put_line('-- ***************************************');
  dbms_output.put_line('-- Start time: '
                       || to_char(sysdate, 'mm/dd/yyyy hh24:mi:ss'));
  dbms_output.put_line(chr(10));

  -- Get all the missing relationships
  open missingMapping;
  loop
    fetch missingMapping into mapping;
    exit when missingMapping%NOTFOUND;

    -- Determine the field associated with the EntityType
    Select FieldID into vFieldID from ISEntityFields
    where EntityTypeID = mapping.EntityTypeID;

    -- Determine the table name
    select ISTables.TableName into vSourceTable from IRFields, ISTables
    where IRFields.TableID = ISTables.TableID
    and IRFields.FieldID = vFieldID;

    -- Determine the increment column name
    select distinct StartFieldName into vSourceColumnID from ISJoins
    where StartTableID in
    (select TableID from IRFields where FieldID = vFieldID);

    -- Determine the source field name
    Select FieldName into vSourceColumnField from IRFields
    where fieldid = vFieldID;

    dbms_output.put_line('insert into  ' || vSourceTable
      || '(' || vSourceColumnID || ', ' || vSourceColumnField || ') Values('
      || to_char(mapping.EntityID) || ', NULL);');

  end loop;
  close missingMapping;

  dbms_output.put_line(chr(10));
  dbms_output.put_line('-- End time: '
                       || to_char(sysdate, 'mm/dd/yyyy hh24:mi:ss'));
  dbms_output.put_line('-- ***************************************');

End;
/
spool off

set echo on
set termout on
set feedback on

spool vra_relation_update_cmd.log
@vra_relation_update_cmd.sql
spool off

commit;

exit

